%% USYD iGEM 2015 Modelling
% This program is designed to predict transcription rate of mRNA molecules,
% based on the models in 'The Dynamics of Supply and Demand in mRNA
% Translation', Brackley, Romano and Thiel, 2011, PLoS Computational
% Biology
% FOr further information see: http://2015.igem.org/Team:Sydney_Australia/Modeling/visualisation

clear %Clear all variables stored
clc %Clear the work space
figure(1) %Open initial figure

%% Simulation Parameters
Codon_number=5e4; %Total number of codons in cell
Codon_to_tRNA_ratio=3.5/3; %Ratio of Codons to tRNA
T_bar=Codon_number/Codon_to_tRNA_ratio; %Total number of RNAs, Constant. 
r=10*41/T_bar; %Coefficient for calculating Ribosome hopping rate 
ku_mean=10; %Hopping rate frequency (s^-1)
k_cat=6.29; %Turnover number (s^-1) assumed same for all codons.
K_m=1.28e18; %Constant K (molecules m^-3);
Cell_volume=2.9e-17; %Volume of a yeast cell (m^3)
K_num=K_m*Cell_volume; %Constant K converted into number of molecules per cell
E_per_tRNA=0.208; %Number of enzymes per tRNA molecule.
ribosome_width=9; %IT is assumed the rightmost covered codon for which the ribosome is awaiting a tRNA
alpha=10; %Ribosome start (hop on) rate (s^-1) Meant to be 10 in the paper
beta=100; %Ribosome end (hop off) rate (s^-1)

%% Read in GCN number files from file, which are used to calculate relative T_u abundances
organism='myco';
[start_organism_standard_rate codon_names codon_ids]=readOrganism([organism,'.csv'],1);
[start_gcn_freq]=readOrganism([organism,'gcn.csv'],0,codon_names);
[start_organism_spencer_rate]=SpencerRateCalculate(start_gcn_freq,codon_names);
Anticodon=findAnticodon(codon_names);
GCN=start_gcn_freq;

%% Now calculate how many GCN's are non zero i.e. how many tRNA species are being produced.
num_produced=sum(start_gcn_freq~=0);
[subtracting,subtracting2]=readSubtracting(start_gcn_freq); %This outputs two vectors which show which tRNA correspond to which codon via Spencer decoding.

%% Initialising the mRNA vector. 
%If we want to use our own mRNA design then here it should be read in.
%Otherwise, we generate one randomly:
mRNA_copies=100 ; %Number of mRNA molecules to simulate
start_num=mRNA_copies; %How many mRNA we will have to begin with if we want to increae the number over time
fname='EtnABCDD';
fid=fopen([fname,'.txt']);
line=fgetl(fid);
fclose(fid);
sequence=line;
[sequence_amino,sequence_codon]=decode_sequence(sequence,codon_names,codon_ids);
[Spencer_rate]=SpencerRateImplementB(sequence_codon,start_organism_spencer_rate,31); %% Calculating the moving average rate.


mRNA_len=length(sequence_codon); %Length of each molecule (in Amino Acids)
mRNA=[sequence_codon 64]; %Make the random mRNA vector
%The plus 1 is to have a final point on the vector from which ribosomes
%drop of with a coefficient rate beta
mRNA2=(   diag(mRNA) *ones(length(mRNA),mRNA_copies))';
%% Initialising calculation parameters
mRNA_occupied=zeros(mRNA_copies,mRNA_len+1);  %mRNA_occupied stores binary data (1's and zeros) corresponding to whether a particular point has a ribosome bound to it
major_size=size(mRNA_occupied); %Recording its size

%% Simulation Parameters
T_u=0*zeros(1,64); %T_u is the number of aa-tRNA of each type
time_range=0:0.01:100; %Setting up time steps for simulation.
plot_freq=20; %How frequenty you want to plot what is goign on, this slows the program down if you set it smaller than ~100!


GCN_ratio=GCN/sum(GCN);
T_bar_u=GCN_ratio*T_bar;%Total number of each tRNA, charged and uncharged, constant.

%% Calculating additional parameters  
E_u=T_bar_u*E_per_tRNA; %Calculating numbers of each enzyme.
V_u=E_u*k_cat;% Maximum charging rate for each synthetase.
R_u=@(T_u,T_bar_u)(V_u.*(T_bar_u-T_u)./(K_num+T_bar_u-T_u)); %Recharging rate of each tRNA from Michaelis menten Equation

%% Defining iteration parameters.
dt=mean(diff(time_range)); %time step for model
T_u_record=zeros(length(time_range),length(T_u)); %Setting up matrix to store results
T_u=0.5*T_bar_u; %Initialising the values for T_u arbitrarily.

initiate_prob=1-exp(-1*alpha*dt);  %Calculate probability for Ribosome to hop on eg. begin translateion
fall_prob=1-exp(-1*beta*dt); %Calculate probability for Ribosome to hop off eg. end translateion

%This convolution business is all for the purpose of making sure that when
%the code is run, Ribosomes cant move forward unless there is a sufficient
%gap between them and the next Ribosome.
Convolution = ones(1,ribosome_width*2+1); %Making a convolution, which is kind of like a running average
Convolution(ribosome_width+1)=0;
Convolution(ribosome_width*2+1)=0;
completed=zeros(mRNA_copies,1);
tic
% vidObj = VideoWriter('test2.mp4','MPEG-4');
% open(vidObj);
for i=1:length(time_range);
   
    Occupied_old=mRNA_occupied;  %Storing which positions were occupied in the previous step.
    Occupied_neighbours=conv2(mRNA_occupied,Convolution,'same')>0; % This matrix will equal 1 everywhere that there is an activated site within 8 on each side
    %Try testing this convolution on a vector of just 1 and zero and see
    %what it does, in theory it is something like a runnign average which
    %will tell us when another ribosome is too close.
    
   
    T_u=dt*R_u(T_u,T_bar_u)+T_u;  %Increment T_u by the amount dictated by Mikaelis Menton Equation
    T_u=T_u.*(T_u>0); %Make sure that these are not less than zer.
    T_u_B=T_u(subtracting2(:,1)).*subtracting2(:,2)'+T_u(subtracting2(:,3)).*subtracting2(:,4)'; %Select correct rates in terms of Spencer rates!
    k_u=r*T_u_B; %Calcultae k_u, the hopping rate for a ribosome searching for T_u
    probability_step=1-exp(-dt*k_u); %Given the rates above we calculate the probabilt for it to jump in time dt.
    
    trans_mat=ones(major_size)*diag(probability_step(mRNA)); %Now put this probability into a large matrix for comparison
    
    transitioning=(rand(major_size)<trans_mat).*mRNA_occupied.*(Occupied_neighbours==0); %Select ribosomes which should transition: Ones where we beat the probability, and there is a ribosome present, and there is no ribosome blocking their way
    
    %Now take care of Ribosomes at the end of the mRNA, they drop off at
    %some given rate
    completing=(rand(mRNA_copies,1)<fall_prob).*(mRNA_occupied(:,mRNA_len)==1);
    mRNA_occupied(:,mRNA_len)=mRNA_occupied(:,mRNA_len)-completing;
    completed(i)=sum(completing); %We want to record how many copies of eacn mRNA are made!
    
    %% Now take care of Ribosomes at the start of the mRNA, they hop on at
    %some given rate, so long as nothing is obstructing their way.
    mRNA_occupied(:,1)=mRNA_occupied(:,1)+(rand(mRNA_copies,1)<initiate_prob).*(Occupied_neighbours(:,1)==0).*(mRNA_occupied(:,1)==0);
    
    %% If we want to do a simulation whereby we start with 1 mRNA and
    %multiply it to see how long this takes, uncomment the following line:

%         mRNA_occupied(min([sum(completed)+1+start_num mRNA_copies]):mRNA_copies,1)= 0;

    %% Finding which ribosomes to move along
    %Ribsomes which we have decided can move this round, remove their
    %previous locations
    mRNA_occupied(:,1:mRNA_len-1)=mRNA_occupied(:,1:mRNA_len-1)-transitioning(:,1:mRNA_len-1);
    %And add their new locations
    mRNA_occupied(:,2:mRNA_len)=mRNA_occupied(:,2:mRNA_len)+transitioning(:,1:mRNA_len-1);
    
    %Find points in which we previously had a ribosome and now there is
    %none. If there was previously a ribosome at a point, now there isn't,
    %the ribosome must have been consumed!!
    consumed=sum(mRNA_occupied(:,1:mRNA_len-1)<Occupied_old(:,1:mRNA_len-1));
    
    %Select the location of ones where this is true
    sel=consumed>0;    
    %We now want to remove the correct number of T_u molecules from the
    %population corresponding to how many were consumed by ribosomes
    T_u_taken=consumed(sel); %Selecting how many were taken
    T_u_ID=mRNA(sel); %Selecting which T_u is responsibel for which mRNA codon 
    for j=1:length(T_u_ID); %Going through each mRNA codon which was advanced
        
         T_u(subtracting(T_u_ID(j),1))= T_u(subtracting(T_u_ID(j),1))-T_u_taken(j)*subtracting(T_u_ID(j),2); %Subtracting the appropriate number from T_u
        T_u(subtracting(T_u_ID(j),3))= T_u(subtracting(T_u_ID(j),3))-T_u_taken(j)*subtracting(T_u_ID(j),4); %Subtracting the appropriate number from T_u wobble factors
    end
   T_u_record(i,:)=T_u./T_bar_u; %Recording for later the number of aa-tRNAs
   
   %If you want to store how many ribosomes are in use un-comment the
   %following line
   %ribosomes_in_use(i)=sum(sum(mRNA_occupied));
   
   
   if mod(i,plot_freq)==1 %Every 50 steps display what is going on
      
       if mRNA_copies==1 %If we are plotting a single mRNA we can do it in a graph that looks nice!
           ON=mRNA_occupied(1:length(mRNA_occupied)-1);
           lox=find(ON);
           plot(Spencer_rate)
           colours={'-r','-m','-k','-b','-g'};
           hold on
           lox=fliplr(lox);
           for j=1:length(lox);
               i=lox(j);
               plotrange=i-round((ribosome_width-1)/2):i+round((ribosome_width-1)/2);
               
               plotrange=plotrange(and((plotrange>0),(plotrange<length(ON))));
           plot(plotrange,Spencer_rate(plotrange),colours{mod(j-1,5)+1},'linewidth',5)
           end
           hold off
            xlabel('Position on mRNA');
            ylabel('Translation Rate (As calculated in TransOpt)')

       else %% Otherwise if we plot many tRNA we do it in an overview plot.
           
             Occupied_neighbours2=conv2(mRNA_occupied,ones(1,ribosome_width),'same')>0;
           imagesc(Occupied_neighbours2+(mRNA2.*(Occupied_neighbours2-1))*-1*0.3/41)
            xlabel('Position on mRNA');
            ylabel('mRNA molecule')
       end

      pause(0.05)
   
%      Uncomment this if you want to create a video
%          currFrame = getframe(gcf);
%    writeVideo(vidObj,currFrame);
    end
    
end
%      Uncomment this if you want to create a video
%close(vidObj);

toc %Output the time this took

figure(2)

plot(time_range(1:40:length(time_range)),T_u_record(1:40:length(time_range),:));
plot(time_range(1:1:length(time_range)),T_u_record(1:1:length(time_range),:));
xlabel('Time (seconds)')
ylabel('Fraction of tRNA population which has amino acid attached')
title('Proportion of T_u that are charged vs Time')

select=mean(T_u_record); %Find mean of each species concentration
[a loc]=sort(select); %Sort them in descendign order
to_plot=loc(1:8); %Pick out ones wihch had lowest mean eg most activity
figure(3)

plot(time_range(1:40:length(time_range)),T_u_record(1:40:length(time_range),to_plot));

xlabel('Time (seconds)')
ylabel('Fraction of tRNA population which has amino acid attached')
title('Eight tRNA from above which had most activity')

disp(['A total of ',num2str(sum(completed)),' copies were produced. Each mRNA was copied on average ',num2str(sum(completed)/mRNA_copies),' times.'])
disp(['On average a new copy of each mRNA is produced each ',num2str(max(time_range)/sum(completed/mRNA_copies)),' seconds'])

